unit TcpMonU;

interface

uses
  IPHelper, IPHLPAPI, TcpObjects, About, LogU,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ToolWin, ComCtrls, ActnList, StdActns, ImgList, StdCtrls,
  ExtCtrls;

type
  TTcpMonF = class(TForm)
    StatusB: TStatusBar;
    ToolBar1: TToolBar;
    ActionL: TActionList;
    ImageL: TImageList;
    FileExit1: TFileExit;
    TcpLB: TListBox;
    StartStopTB: TToolButton;
    ToolButton2: TToolButton;
    RTimer: TTimer;
    StartStopA: TAction;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    NumericPortA: TAction;
    ListeningA: TAction;
    LoopbackA: TAction;
    ToolButton5: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    AboutA: TAction;
    ToolButton10: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    HeaderP: TPanel;
    LogA: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StartStopAExecute(Sender: TObject);
    procedure RTimerTimer(Sender: TObject);
    procedure NumericPortAExecute(Sender: TObject);
    procedure ListeningAExecute(Sender: TObject);
    procedure LoopbackAExecute(Sender: TObject);
    procedure TcpLBDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure AboutAExecute(Sender: TObject);
    procedure LogAExecute(Sender: TObject);
  private
    FStarted: boolean;
    TcpRowList : TTcpList;
    FShowListening: boolean;
    FShowLoopback: boolean;
    FFontSize: integer;
    procedure SetStarted(const Value: boolean);
    procedure OnGetRow( Sender: TObject; TcpRow : TTcpRow; NewRow : boolean);
    procedure ProcessTcpRows( const CheckNew : boolean = true);
    procedure SetShowListening(const Value: boolean);
    procedure SetShowLoopback(const Value: boolean);
    function FormatTcpStr( TR : TTcpRow) : string;
    procedure SetFontSize(const Value: integer);
  public
    property Started : boolean read FStarted write SetStarted;
    property ShowListening : boolean read FShowListening write SetShowListening;
    property ShowLoopback : boolean read FShowLoopback write SetShowLoopback;
    property FontSize : integer read FFontSize write SetFontSize;
  end;

var
  TcpMonF: TTcpMonF;

implementation

{$R *.dfm}

procedure TTcpMonF.FormCreate(Sender: TObject);
begin
  TcpRowList := TTcpList.Create;
  TcpRowList.OnGetRow := OnGetRow;

  FStarted := false;
  if LoadIpHlp then begin
    Started := true;
    end
  else ShowMessage( 'Internet Helper DLL Not Available or Not Supported') ;

  FShowListening := true; ShowListening := false;
  FShowLoopback := true; ShowLoopback := false;
end;

procedure TTcpMonF.FormDestroy(Sender: TObject);
begin
  TcpRowList.Free;
end;


procedure TTcpMonF.ProcessTcpRows(const CheckNew: boolean);
var
  AreNewRows : boolean;
  Cou : integer;
  TR : TTcpRow;
  TCPStats      : TMibTCPStats;
  ErrorCode     : DWORD;
begin
  // Get TCP statistics
  ErrorCode := GetTCPStatistics( @TCPStats );
  if ErrorCode = NO_ERROR then begin
    StatusB.Panels[1].Text := 'Established: ' + IntToStr( TCPStats.dwCurrEstab);
    StatusB.Panels[2].Text := 'Segments In/Out: ' +
                             IntToStr( TCPStats.dwInSegs) + ' / ' +
                             IntToStr( TCPStats.dwOutSegs);
    end;

  // Update Tcp connections list
  AreNewRows := TcpRowList.Refresh;
  if CheckNew AND ( NOT AreNewRows) then exit;
  TcpLB.Items.Clear;
  for Cou := 0 to TcpRowList.Count - 1 do begin
    TR := TcpRowList.Items[ Cou];
    TcpLB.Items.AddObject( FormatTcpStr( TR), TR);
    end;
end;

procedure TTcpMonF.OnGetRow(Sender: TObject; TcpRow: TTcpRow;
  NewRow: boolean);
begin
  if Assigned( LogsF) then LogsF.LogAdd( FormatTcpStr( TcpRow));
end;

procedure TTcpMonF.SetStarted(const Value: boolean);
begin
  if FStarted = Value then exit;
  FStarted := Value;
  if FStarted then begin
    StatusB.Panels[0].Text := 'Started at ' + DateTimeToStr( Now);
    end
  else begin
    StatusB.Panels[0].Text := 'Stoped';
    StatusB.Panels[1].Text := '';
    StatusB.Panels[2].Text := '';
    end;
  RTimer.Enabled := FStarted;
end;

procedure TTcpMonF.StartStopAExecute(Sender: TObject);
begin
  StartStopA.Checked := NOT StartStopA.Checked;
  if StartStopA.Checked then begin
    StartStopA.ImageIndex := 1;
    StartStopA.Hint := 'Stop';
    end
  else begin
    StartStopA.ImageIndex := 2;
    StartStopA.Hint := 'Start';
    end;
  Started := StartStopA.Checked;
end;

procedure TTcpMonF.RTimerTimer(Sender: TObject);
begin
  RTimer.Enabled := false;
  try
    ProcessTcpRows;
  finally RTimer.Enabled := true; end;
end;

procedure TTcpMonF.NumericPortAExecute(Sender: TObject);
begin
  NumericPortA.Checked := NOT NumericPortA.Checked;
  ProcessTcpRows( false);
end;

procedure TTcpMonF.ListeningAExecute(Sender: TObject);
begin
  ListeningA.Checked := NOT ListeningA.Checked;
  ShowListening := ListeningA.Checked;
end;

procedure TTcpMonF.SetShowListening(const Value: boolean);
begin
  if FShowListening = Value then exit;
  FShowListening := Value;
  if FShowListening then TcpRowList.StateFilter := []
  else                   TcpRowList.StateFilter := [ tcpfListening];
  ProcessTcpRows( false);
end;

procedure TTcpMonF.LoopbackAExecute(Sender: TObject);
begin
  LoopbackA.Checked := NOT LoopbackA.Checked;
  ShowLoopback := LoopbackA.Checked;
end;

procedure TTcpMonF.SetShowLoopback(const Value: boolean);
begin
  if FShowLoopback = Value then exit;
  FShowLoopback := Value;
  TcpRowList.ShowLoopback := FShowLoopback;
  ProcessTcpRows( false);
end;

procedure TTcpMonF.TcpLBDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  LB : TListBox;
  R : TRect;
  TR : TTcpRow;
  IIndex : integer;

  procedure DrawColorRect( Rect : TRect; State : cardinal);
  var
    TmpColor : TColor;
    TmpBrush : TBrush;
    TmpRect : TRect;                    
  begin
    TmpColor := clGray;
    case TTcpStateFilter( State) of
      tcpfNone        : TmpColor := clGray;
      tcpfClosed      : TmpColor := clGray;
      tcpfListening   : TmpColor := clYellow;
      tcpfSyn_sent    : TmpColor := clGreen;
      tcpfSyn_rcvd    : TmpColor := clGreen;
      tcpfEstablished : TmpColor := clLime;
      tcpfFin_wait1   : TmpColor := clMaroon;
      tcpfFin_wait2   : TmpColor := clMaroon;
      tcpfClose_wait  : TmpColor := clRed;
      tcpfClosing     : TmpColor := clRed;
      tcpfLast_ack    : TmpColor := clRed;
      tcpfTime_wait   : TmpColor := clMaroon;
      tcpfDelete_tcb  : TmpColor := clGray;
      end;
    TmpBrush := LB.Canvas.Brush;
    LB.Canvas.Brush.Color := TmpColor;
    LB.Canvas.Brush.Style := bsSolid;
    TmpRect := Rect; TmpRect.Right := TmpRect.Left + TmpRect.Bottom - TmpRect.Top;
    inc( TmpRect.Top); dec( TmpRect.Bottom);
    LB.Canvas.FillRect( TmpRect);
  end;

  procedure CenterText(Cnv: TCanvas; Rect: TRect; S: string);
  var X, Y: Integer;
  begin
    X := Rect.Left;
    Y := (Rect.Bottom + Rect.Top - Cnv.TextHeight(S)) div 2;
    Cnv.TextOut(X, Y, S);
  end;
  
begin
  LB := Control as TListBox;
  R := Rect;
  LB.Canvas.FillRect( Rect);
  R.Left   := R.Left  + 18;
  CenterText( LB.Canvas, R, LB.Items.Strings[Index]);

  TR := LB.Items.Objects[Index] as TTcpRow;
  if TR <> NIL then begin
    IIndex := TR.StateDW;
    DrawColorRect( Rect, IIndex);
    end;
end;

procedure TTcpMonF.AboutAExecute(Sender: TObject);
begin
  AboutBox := TAboutBox.Create( NIL);
  AboutBox.ShowModal;
  AboutBox.Free;
end;

procedure TTcpMonF.LogAExecute(Sender: TObject);
begin
  LogsF.Show;
end;

function TTcpMonF.FormatTcpStr(TR: TTcpRow): string;
begin
  Result := TimeToStr( TR.Timestamp) + ' | ' +
    TR.LocalAddr  + ':' + TR.PortName( TR.LocalPortDW, NumericPortA.Checked) + ' | ' +
    TR.RemoteAddr + ':' + TR.PortName( TR.RemotePortDW, NumericPortA.Checked) + ' | ' +
    TR.State;
end;

procedure TTcpMonF.SetFontSize(const Value: integer);
begin
  if FFontSize = Value then exit;
  FFontSize := Value;
  TcpLB.Font.Size := FFontSize;
  HeaderP.Font.Size := FFontSize;
end;

end.
